/********************************************************************
 * 	asc0.c
 *
 *	Alle Funktionen fr die serielle Schnittstelle.
 *  
 *	
 ********************************************************************/
#include "asc0.h"
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

//extern int	memory[65536];		/* the memory is global */

#ifndef DONT_USE_ASC0

	/* definitions for the serial receive/transmit pins */
	sbit DP310 = DP3^10;
	sbit DP311 = DP3^11;
	sbit P310 = P3^10;
	sbit P311 = P3^11;

	/********************************************************************
	 * asc0_put
	 *
	 *	Print one char to asc. 
	 *	If another is still in queue then wait untill it is sent.
     *
	 *  !! For a more exhaustive version see C166\lib\putchar.c !!
	 *
	 *	Input:	c		character to send
	 *  Return: -
	 ********************************************************************/
	void
	asc0_put(uchar c)
	{
//		SOTIR = 1;
		while (! S0TIR);
		S0TIR = 0;
	
		S0TBUF = c;
	
		return;
	}
	
	/********************************************************************
	 * asc0_get
	 *
	 *	Read one char from asc.
	 *	If none is avail, wait untill one is received.
     *
     *  !! Also take a look at the file C166\lib\getkey.c !!
	 *
	 *  Input:  -
	 *	Return:	character received
	 ********************************************************************/
	uchar
	asc0_get()
	{
//		S0RIR = 1;		
		while (! S0RIR);
		S0RIR = 0;
	
		return S0RBUF;
	
	}
	
	/********************************************************************
	 * asc0_printf 
	 *
	 *	Print a string to asc.
	 *	Call is identic to normal printf.
	 *
	 *	Input: fmt		Format specifier
	 *		   ...		any parameters referenced by fmt
	 *	Return: -
	 ********************************************************************/
	void
	asc0_printf(char *fmt, ...)
	{
		uchar buffer[256];
		va_list arg_ptr;
		uint i;
	
		va_start(arg_ptr, fmt);
		vsprintf(buffer, fmt, arg_ptr);
		va_end(arg_ptr);
	
		for (i = 0; buffer[i]; i++)
			asc0_put(buffer[i]);
	
	 	return;
	}	
	
	/********************************************************************
	 * init_asc0
	 *
	 *	Initialise asynchonous serial interface.
	 *		9600Bd, 8,1,N 
	 ********************************************************************/
	void 
	init_asc0(void) 
	{
		P310 = 1;		    			/* Set Transmit Pin      */
		DP310 = 1;					    /* Direction -> Transmit */
		P311 = 0;						/* Clear Receive Pin     */
		DP311 = 0;					    /* Direction -> Receive  */
	
	    /* 
		   Set Interruptlevels:
		   S0TIC  ... ASC0 Transmit Interrupt,
		   S0TBIC ... ASC0 Transmit Buffer Interrupt,
		   S0RIC  ... ASC0 Receiver Interrupt,
		   S0EIC  ... ASC0 Error Interrupt 
	    
	    S0TIC = 0x55;			
		S0TBIC = 0x56;			
	 	S0RIC = 0x57;			
		*/
	
		/* 
		   Set the Baudrate Register S0BG: (exact value)
	       0x0004 ... 115200 Baud, (125000.000)
	       0x000A ...  57600 Baud, ( 56818.128)
	       0x000F ...  38400 Baud, ( 39061.500)
	       0x0014 ...  28800 Baud, ( 29761.905)
	       0x0020 ...  19200 Baud,
	       0x0040 ...   9600 Baud 
	    */
		S0BG = 0x0040;
	
		/* Set ASC0 Mode: 8 data, no parity, 1 Stop Bit */
		S0CON = 0x0001;
	
		/* Receiver Enable */
		S0REN = 1;
	
		/* Enable Baud Rate Generator */
		S0R = 1;
	
		/* Set Transmit Interrupt Flag */
		S0TIR = 1;				
			
		return;
	}	


	/********************************************************************
	 * void softdelay (unsigned int value)
	 *
	 * 		Timer Funktion
	 * 
	 ********************************************************************/


		void softdelay (unsigned int value) 
		{
		unsigned int i,j;
			for (j=0; j<value; j++)
			{
			for (i=0; i< 0x20; i++)
				{
				}
			}
		}

#else
	
	/********************************************************************
	 * In case no I/O functions are required!
	 ********************************************************************/
	void init_asc0 (void) {}
	void asc0_put(uchar c) {}
	uchar asc0_get() { return 0; }
	void asc0_printf(char *fmt, ...) {}
    void softdelay (unsigned int);

#endif


