/******************************************************
 * 	flash.c
 * 
 * 	Funktionen fr die Flash-Speicher Befehle
 * 
 ******************************************************/

#include "flash.h"
#include "asc0.h"


	/********************************************************************
	 * 	reset()
	 *
	 *	Flash-Speicher vor dem Schreiben resetten.
	 *
	 ********************************************************************/

	void reset()
	{
		*(unsigned int volatile far *) (FBASE+(0x23 << 1)) = 0xF0F0;
	}


	/********************************************************************
	 * 	clear()
	 *
	 *	Lscht den Flash-Speicher.
	 *
	 ********************************************************************/

	void clear()
 	{
	 	*(unsigned int volatile far *) (FBASE+(0x555 << 1)) = 0xAAAA;
		*(unsigned int volatile far *) (FBASE+(0x2AA << 1)) = 0x5555;
		*(unsigned int volatile far *) (FBASE+(0x555 << 1)) = 0x8080;
		*(unsigned int volatile far *) (FBASE+(0x555 << 1)) = 0xAAAA;
		*(unsigned int volatile far *) (FBASE+(0x2AA << 1)) = 0x5555;
		*(unsigned int volatile far *) (FBASE+(0x555 << 1)) = 0x1010;
 	}

	/********************************************************************
	 * 	write(unsigned int addr, unsigned int data)
	 *
	 *	Funktion um Daten (data) in den Flash-Speicher an eine bestimmte
	 *	Adresse (addr) zu schreiben.
     *
	 * 
	 ********************************************************************/

	void write(unsigned int addr, unsigned int data)
	{
		//asc0_printf("%d: Try to write %#X to address %ld\n",0,data,(addr&0x1FFFFE));
 		*(unsigned int volatile far *) (FBASE+(0x555 << 1)) = 0xAAAA;
		*(unsigned int volatile far *) (FBASE+(0x2AA << 1)) = 0x5555;
		*(unsigned int volatile far *) (FBASE+(0x555 << 1)) = 0xA0A0;
		*(unsigned int volatile far *) (FBASE+(addr & 0x1FFFFE)) = data;
	}


	/********************************************************************
	 * 	unsigned int read(unsigned int addr)
	 *
	 *	Funktion um Daten aus den Flash-Speicher aus einer bestimmten
	 *	Adresse (addr) zu lesen.
     *
	 * 
	 ********************************************************************/

	unsigned int read(unsigned int addr)
	{
		return *(unsigned int volatile far *) (FBASE+(addr << 1));
	}


	/********************************************************************
	 * 	check()
	 *
	 *	Diese Funktion schaut, ob der Flash-Speicher schon bereit ist.
	 *
	 ********************************************************************/

	int check() 
	{
		unsigned int temp;
		int test = 1;
		do
			{
			temp = *(unsigned int volatile far *) (FBASE+(0x23 << 1)); 
			temp = temp & 0x2020;	
			}
		while(temp != 0x2020);	
		temp = *(unsigned int volatile far *) (FBASE+(0x23 << 1)); 
		if((temp & 0x8080)!= 0x8080) test = 1;
		else test = 0; 
		reset();
		return test;
	}

