/* -*- mode: c; compile-command: "cd ..; make all"; -*- */
/*
 * Module:         Functions to support C-Programms
 * Author:         Guenter Ebermann
 * Matrikelnummer: tw01d022
 * Contents:       Module that holds functions for C-Programms
 * Time-stamp:     <2003-01-21 23:10:17 guenter>
 * File:           support.c
 */

#include <stdlib.h> /* For strtol */
#include <stdio.h>  /* For *printf */
#include <errno.h>  /* For errno */
#include <string.h> /* For strerror */
#include <ctype.h> /* For isascii */
#include <stdarg.h> /* For va_list */

#include "flashtool.h" /* always include this file as first one - it is the global include file */
#include "error_handling.h"

void _print_error (ERROR_TYPE error_type, const char *format, const va_list ap);

/* Extern functions */
void free_resources (void);

/* Extern variables */
extern const char *command_name;

/*
 * Function _print_error
 * Prints error message on stderr or stdout
 * Parameters: const char *format
 * Return values: void
 * Uses global variables: command_name
 */
void _print_error (ERROR_TYPE error_type, const char *format, const va_list ap) 
{
  char message[MESSAGE_LEN];
  
  vsnprintf (message, MESSAGE_LEN, format, ap);

  if (error_type == MESSAGE)
    printf("%s: %s\n", command_name, message);
  else
    {
      if (errno != 0) 
	{
	  if (error_type == WARNING)
	    fprintf(stderr, "%s: warning: %s - %s\n", command_name, message, strerror(errno) );
	  else
	    fprintf(stderr, "%s: %s - %s\n", command_name, message, strerror(errno) );
	}
      else 
	{
	  if (error_type == WARNING)
	    fprintf(stderr, "%s: warning: %s\n", command_name, message );
	  else
	    fprintf(stderr, "%s: %s\n", command_name, message );
	}
    }
  fflush (NULL); /* flush all puffers */
}

void print_message (ERROR_TYPE error_type, const char *format, ...)
{
  va_list ap;
  va_start (ap, format);
  
  if (format != (const char *) 0 || error_type == BAIL_OUT)
    {
      switch (error_type) 
	{
	case MESSAGE:
	case BAIL_OUT:
	case WARNING:
	  _print_error (error_type, format, ap);
	  break;
	default:
	  _print_error (WARNING, "False invocation of print_message", 0);
	  break;
	}
      va_end (ap);
      if (error_type==BAIL_OUT)
	{
	  free_resources();
	  exit (EXIT_FAILURE);
	}      
    }
}

void error_detection (bool verbose, int result,  RESULT_QUALIFIER result_qualifier, const char *format, ...)
{
  char message[MESSAGE_LEN];

  va_list ap;
  va_start (ap, format);
  
  vsnprintf (message, MESSAGE_LEN, format, ap);

  switch (result_qualifier)
    {
    case NORMAL:
      if (result <0)
	print_message (BAIL_OUT, "failure %s", message);
      else if (verbose)
	print_message (MESSAGE, "successfully %s", message);
      break;
    case NORMAL_WARNING:
      if (result <0)
	print_message (WARNING, "failure %s", message);
      else if (verbose)
	print_message (MESSAGE, "successfully %s", message);
      break;
    case NULLONE:
      if (result == 0)
	print_message (BAIL_OUT, "failure %s", message);
      else if (verbose)
	print_message (MESSAGE, "successfully %s", message);
      break;
    case NORESULT:
      if (verbose)
	print_message (MESSAGE, "%s", message);
      break;
    case NORESULT_BAILOUT:
      print_message (BAIL_OUT, "%s", message);
      break;
    default:
      print_message (WARNING, "false invocation of error_detection", 0);
    }

      va_end (ap);
}

/*
 * Funktion is_ascii
 * determine if string is made of ascii characters only 
 * Parameter: const char *string
 * Return Werte: int 
 *               -1 if not an ascii string
 *               0 if an ascii string
 * Benutzt globale Variablen: none
 */
int is_ascii (const char *string)
{
  int ret = 0;
  int length = 0;
  int i = 0;
  
  length = strlen (string);
  
  for ( i=0; i<length; i++)
    {
      ret = isascii(*(string+i));
      if (ret == 0)
	return -1;
    }
  return 0;      
}

void print_version (void)
{
  puts ("Flashtool 1.0");
  puts ("Copyright (C) 2003 Guenter Ebermann");
  puts ("Flashtool comes with ABSOLUTELY NO WARRANTY.");
  puts ("You may redistribute copies of Flashtool");
  puts ("under the terms of the GNU General Public License.");
}

void print_help (void)
{
  printf("Flashtool invocation: ./flashtool [OPTIONS] INFILE\n");
  /* printf("general options:\n"); */
  printf("--output -o [FILENAME] output in file\n");
  printf("--verbose              print more output\n");
  /* printf("serial setup:\n"); */
  printf("--noinit               no init of serial line - use stty\n");
  printf("--baud -b [BAUDRATE]   use baudrate eg. 57600\n");
  printf("--device -d [DEVICE]   use device eg. /dev/ttyS0\n");
  /* printf("converting functions:\n"); */
  printf("--convert              convert from intel hex to bin\n");
  printf("--convert3             convert from intel hex to bin for 3rd bsl\n");
  printf("--hextobin             convert raw hex to bin\n");
  /* printf("bootstrap:\n"); */
  printf("--init                 only send the 0 byte\n");
  printf("--bootstrap            transfer the bootstrap loaders (1st, 2nd and 3rd)\n");
  printf("--bootstrap3           transfer 3rd bootstrap loader\n");
  printf("--write                automates the downloading of data\n");
  /* communictation with bsl3 */
  printf("--flash                write ihex on eprom\n");
  printf("--delete               delete eprom\n");
  printf("--read                 read from flash memory\n");
  printf("--start -s [ADDRESS]   only with --read\n");
  printf("--stop -t [ADDRESS]    only with --read\n");
  printf("--info                 print info of connected board an processor\n");
  /* printf("utilities:\n"); */
  printf("--terminal             serial terminal programm - talk over serial line\n");
  /* printf("standard switches: \n"); */
  printf("--version              print version number and exit\n");
  printf("--help                 print this help and exit\n");
  printf("Report bugs to guenter.e@gmx.at\n");      
}

